$(document).ready(function(){
    $('[data-toggle="tooltip"]').tooltip();
    var valid_operators = {
        "addition": `<i class="fas fa-plus"></i>`,
        "substraction": `<i class="fas fa-minus"></i>`,
        "multiplication": `<i class="fas fa-times"></i>`,
        "division": `<i class="fas fa-divide"></i>`
    };

    var bangla_to_english_mapper = {
        '০': "0",
        '১': "1",
        '২': "2",
        '৩': "3",
        '৪': "4",
        '৫': "5",
        '৬': "6",
        '৭': "7",
        '৮': "8",
        '৯': "9"
    };

    var english_to_bangla_mapper = {
        "0": '০',
        "1": '১',
        "2": '২',
        "3": '৩',
        "4": '৪',
        "5": '৫',
        "6": '৬',
        "7": '৭',
        "8": '৮',
        "9": '৯'
    };

    var quiz_data = [];
    var quiz_settings = {};
    $('#celebration_deck').hide();

    function display_celebration_effects(){
        var end = Date.now() + (5 * 1000);
        // go Buckeyes!
        var colors = ['#FF0000', '#093657', '#D7B030'];
        (function frame() {
            confetti({
                particleCount: 3,
                angle: 60,
                spread: 55,
                origin: { x: 0 },
                colors: colors,
                disableForReducedMotion: true
            });
            confetti({
                particleCount: 3,
                angle: 120,
                spread: 55,
                origin: { x: 1 },
                colors: colors,
                disableForReducedMotion: true
            });

            if (Date.now() < end) {
                requestAnimationFrame(frame);
            }
        }());
    }

    function display_celebration_message(){
        $('#celebration_deck').slideDown("slow");
    }

    function celebrate(){
        display_celebration_effects();
        display_celebration_message();
    }

    function get_random_integer(min_number, max_number){
        return Math.floor(Math.random() * (max_number - min_number + 1) + min_number);
    }

    function get_random_operator(operators){
        operator = operators[Math.floor(Math.random() * operators.length)]
        if(valid_operators.hasOwnProperty(operator)){
            return valid_operators[operator];
        }
        valid_operators_keys = Object.keys(valid_operators);
        var random_index = Math.floor(Math.random() * valid_operators_keys.length);
        random_operator = valid_operators[valid_operators_keys[random_index]];
        return random_operator;
    }

    function get_converted_value(original_value, language_code){
        mapper = "";
        switch (language_code) {
            case "en":
            mapper = bangla_to_english_mapper;
            break;
            default:
            mapper = english_to_bangla_mapper;
        }
        converted_value = [];
        for(i = 0; i < original_value.length; i++){
            character = original_value[i];
            if(mapper.hasOwnProperty(character)){
                converted_value.push(mapper[character]);
            }
            else{
                converted_value.push(character);
            }
        }
        return converted_value.join("");
    }

    function get_correct_answer(operand_1, operand_2, operator){
        correct_answer = NaN;
        switch (operator) {
            case valid_operators["addition"]:
            correct_answer = operand_1 + operand_2;
            break;
            case valid_operators["substraction"]:
            correct_answer = operand_1 - operand_2;
            break;
            case valid_operators["multiplication"]:
            correct_answer = operand_1 * operand_2;
            break;
            case valid_operators["division"]:
            correct_answer = Math.floor(operand_1 / operand_2);
            break;
            default:
            break;
        }
        return correct_answer;
    }

    function get_a_quiz_row(question){
        return `<tr class="quiz_row bg-dark text-white text-center align-items-center">
        <td>
        <!-- <img src="icon/mat-cuoi.png" width="50" height="50" alt="Smiles icon"> -->
        ${question["operand_1"]}
        </td>
        <td>
        ${question["operator"]}
        </td>
        <td>
        <!-- <img src="icon/trai-tim.png" width="50" height="50" alt="Heart icon"> -->
        ${question["operand_2"]}
        </td>
        <td>
        <i class="fas fa-equals"></i>
        </td>
        <td>
        <input id="${question["question_id"]}"
        class="answer_input form-control"
        type="${question["input_type"]}" tabindex="${question["question_id"]+1}">
        </td>
        <td>
        <button class="btn btn-light"><i class="fa fa-2x fa-calculator"></i></button>
        </td>
        </tr>`;
    }

    function get_quiz_settings(){
        $max_number_operand_1 = parseInt($("#max_number_operand_1").val());
        if($max_number_operand_1 == NaN){
            $max_number_operand_1 = 30;
        }
        $max_number_operand_2 = parseInt($("#max_number_operand_2").val());
        if($max_number_operand_2 == NaN){
            $max_number_operand_2 = 10;
        }
        $number_of_questions = parseInt($("#number_of_questions").val());
        $quiz_language = $("#quiz_language").val();
        $type_of_questions = $("#type_of_questions").val();
        if($type_of_questions.length === 0){
            $type_of_questions = ["random"];
        }
        return {
            "max_number_operand_1": $max_number_operand_1,
            "max_number_operand_2": $max_number_operand_2,
            "number_of_questions": $number_of_questions,
            "type_of_questions": $type_of_questions,
            "quiz_language": $quiz_language
        };
    }

    function get_quiz_data(){
        data = [];
        operators = quiz_settings["type_of_questions"];
        max_number_operand_1 = quiz_settings["max_number_operand_1"];
        max_number_operand_2 = quiz_settings["max_number_operand_2"];
        min_number = 0;
        for(var i = 0; i < quiz_settings["number_of_questions"]; i++){
            var question = {};
            var operand_1 = get_random_integer(min_number, max_number_operand_1);
            var operand_2 = get_random_integer(min_number, max_number_operand_2);
            var operator = get_random_operator(operators);
            // Handle Division by zero error
            if(operator == valid_operators["division"]){
                if(operand_2 == 0){
                    operand_2 = 1;
                }
            }
            question["correct_answer"] = get_correct_answer(operand_1, operand_2, operator);
            question["operand_1"] = operand_1;
            question["operand_2"] = operand_2;
            question["input_type"] = "number";
            if(quiz_settings["quiz_language"] === "bn"){
                question["operand_1"] = operand_1.toLocaleString("bn-BD");
                question["operand_2"] = operand_2.toLocaleString("bn-BD");
                question["input_type"] = "text";
            }
            question["operator"] = operator;
            question["question_id"] = i;
            data.push(question);
        }
        return data;
    }

    function getRandSound(auArr, cr) {
        let path = window.location.origin;
        let dirParts = '/audio/';
        if (cr == 'correct') {
            dirParts += 'correct/';
        } else if (cr == 'wrong') {
            dirParts += 'wrong/';
        }
        path += window.location.pathname.slice(0,
            window.location.pathname.lastIndexOf('/')) + dirParts;
        let randElement = auArr[(Math.floor(Math.random() * auArr.length))];
        return path + randElement + '.mp3';
    }

    function correctAns(){
        const auArr = ['ban-lam-dung-roi','ban-lam-tot-lam',
            'ban-rat-thong-minh','be-gioi-qua','be-lam-tot-lam',
            'diem-10-cho-be','gioi-lam','hay-qua','toi-yeu-ban','tuyet-voi-qua'];
        let audioObj = document.createElement('audio');
        audioObj.src = getRandSound(auArr, 'correct');
        audioObj.play();
    }

    function wrongAns() {
        const auArr = ['be-chiu-kho-hoc-nhe','be-co-gang-nhe',
            'be-lam-sai-roi','be-thu-lai-nao','be-thu-lai-nhe',
            'co-gang-len-nhe','gan-dung-roi','o-sai-roi','sai-roi-be-oi'];
        let audioObj = document.createElement('audio');
        audioObj.src = getRandSound(auArr, 'wrong');
        audioObj.play();
    }

    $("#generate_quiz").on("click", function(){
        quiz_settings = get_quiz_settings();
        $("#quiz_wrapper").removeClass("invisible");
        $("#quiz_settings_box").collapse('hide');
        quiz_data = get_quiz_data();
        $("#quiz_rows").html("");
        for(i = 0; i < quiz_data.length; i++){
            $("#quiz_rows").append(get_a_quiz_row(quiz_data[i]));
        }
        const image1 = '<img src="icon/mat-cuoi.png" width="50" height="50" alt="Smiles icon">';
        const image2 = '<img src="icon/trai-tim.png" width="50" height="50" alt="Heart icon">';
        $("#quiz_rows tr td:first-child").prepend(image1);
        $("#quiz_rows tr td:nth-child(3)").prepend(image2);
        $("#celebration_message").html("Very good!");
        $("#correct_answers_title").html("Correct Answer");
        $("#wrong_answers_title").html("Wrong Answer");
        $("#correct_answers_value").html(0);
        $("#wrong_answers_value").html(0);
        if(quiz_settings["quiz_language"] === "bn"){
            $("#celebration_message").html("খুব ভালো!");
            $("#correct_answers_title").html("সঠিক উত্তর");
            $("#wrong_answers_title").html("ভুল উত্তর");
            $("#correct_answers_value").html(get_converted_value("0", "bn"));
            $("#wrong_answers_value").html(get_converted_value("0", "bn"));
        }
        $(".answer_input")[0].focus();
        $('#celebration_deck').hide();
    });

    $('body').on('click', 'button.btn-light', function(event) {
        const ans = $(this).parent().prev().children('input.answer_input');
        current_value = get_converted_value(ans.val(), "en");
        question_id = parseInt(ans.attr("id"));
        correct_anwer = quiz_data[question_id]["correct_answer"].toString();
        total_correct_old = $(".quiz_row.bg-success").length;
        total_wrong_old = $(".quiz_row.bg-danger").length;
        if(current_value === correct_anwer){
            ans.closest(".quiz_row").removeClass("bg-dark");
            ans.closest(".quiz_row").removeClass("bg-danger");
            ans.closest(".quiz_row").addClass("bg-success");
            correctAns();
        }
        else{
            $("#total_questions_value").html(quiz_settings["number_of_questions"]);
            ans.closest(".quiz_row").removeClass("bg-dark");
            ans.closest(".quiz_row").removeClass("bg-success");
            ans.closest(".quiz_row").addClass("bg-danger");
            wrongAns();
        }
        total_correct_new = $(".quiz_row.bg-success").length;

        if(total_correct_new == quiz_data.length){
            celebrate();
        }

        total_wrong_new = $(".quiz_row.bg-danger").length;
        if(total_correct_new !== total_correct_old){
            $("#correct_answers_card").fadeOut();
            $("#correct_answers_value").html(total_correct_new);
            $("#correct_answers_card").fadeIn();
        }
        if(total_wrong_new !== total_wrong_old){
            $("#wrong_answers_card").fadeOut();
            $("#wrong_answers_value").html(total_wrong_new);
            $("#wrong_answers_card").fadeIn();
        }
        if(quiz_settings["quiz_language"] === "bn"){
            $("#correct_answers_value").html(get_converted_value(total_correct_new.toString(), "bn"));
            $("#wrong_answers_value").html(get_converted_value(total_wrong_new.toString(), "bn"));
        }
    });
/*
    $("body").on("focusout", "input.answer_input", function(event){
        current_value = get_converted_value($(this).val(), "en");
        question_id = parseInt($(this).attr("id"));
        correct_anwer = quiz_data[question_id]["correct_answer"].toString();
        total_correct_old = $(".quiz_row.bg-success").length;
        total_wrong_old = $(".quiz_row.bg-danger").length;
        if(current_value === correct_anwer){
            $(this).closest(".quiz_row").removeClass("bg-dark");
            $(this).closest(".quiz_row").removeClass("bg-danger");
            $(this).closest(".quiz_row").addClass("bg-success");
        }
        else{
            $("#total_questions_value").html(quiz_settings["number_of_questions"]);
            $(this).closest(".quiz_row").removeClass("bg-dark");
            $(this).closest(".quiz_row").removeClass("bg-success");
            $(this).closest(".quiz_row").addClass("bg-danger");
        }
        total_correct_new = $(".quiz_row.bg-success").length;

        if(total_correct_new == quiz_data.length){
            celebrate();
        }

        total_wrong_new = $(".quiz_row.bg-danger").length;
        if(total_correct_new !== total_correct_old){
            $("#correct_answers_card").fadeOut();
            $("#correct_answers_value").html(total_correct_new);
            $("#correct_answers_card").fadeIn();
        }
        if(total_wrong_new !== total_wrong_old){
            $("#wrong_answers_card").fadeOut();
            $("#wrong_answers_value").html(total_wrong_new);
            $("#wrong_answers_card").fadeIn();
        }
        if(quiz_settings["quiz_language"] === "bn"){
            $("#correct_answers_value").html(get_converted_value(total_correct_new.toString(), "bn"));
            $("#wrong_answers_value").html(get_converted_value(total_wrong_new.toString(), "bn"));
        }
    });
*/
    $("body").on("keyup", "input.answer_input", function(event){
        current_value = get_converted_value($(this).val(), "en");
        question_id = parseInt($(this).attr("id"));
        total_anwer_input = $(".answer_input").length;
        switch (event.which) {
            case 37:
            prev_question = question_id-1;
            if(prev_question==-1){
                $("#"+question_id).blur();
            }
            else{
                $("#"+prev_question).focus();
            }
            break;
            case 39:
            next_question = question_id+1;
            if(next_question==total_anwer_input){
                $("#"+question_id).blur();
            }
            else{
                $("#"+next_question).focus();
            }
            break;
            default:
            break;
        }
        if(quiz_settings["quiz_language"] === "bn"){
            bangla_value = get_converted_value(current_value, "bn");
            $(this).val(bangla_value);
        }
    });
});
